@@ -14,18 +14,18 @@
 
 #include <KIO/OpenUrlJob>
 #include <KIO/JobUiDelegate>
-#include <KMimeTypeTrader>
 #include <KMessageBox>
 #include <KParts/ReadOnlyPart>
 #include <KParts/BrowserInterface>
 #include <KParts/BrowserExtension>
 #include <KParts/PartLoader>
-#include <KParts/BrowserOpenOrSaveQuestion>
 #include <KIO/FileCopyJob>
+#include <KIO/MimeTypeFinderJob>
 #include <KJobWidgets>
 #include <KProtocolManager>
 #include <KDesktopFile>
 #include <KApplicationTrader>
+#include <KParts/PartLoader>
 
 #include <QDebug>
 #include <QArgument>
@@ -40,8 +40,8 @@ bool UrlLoader::embedWithoutAskingToSave(const QString &mimeType)
     if (s_mimeTypes.isEmpty()) {
         QStringList names{QStringLiteral("kfmclient_html"), QStringLiteral("kfmclient_dir"), QStringLiteral("kfmclient_war")};
         for (const QString &name : names) {
-            KService::Ptr s = KService::serviceByDesktopName(name);
-            s_mimeTypes.append(s->mimeTypes());
+            KPluginMetaData md = findPartById(name);
+            s_mimeTypes.append(md.mimeTypes());
         }
         //The user may want to save xml files rather than embedding them
         //TODO: is there a better way to do this?
@@ -56,7 +56,8 @@ bool UrlLoader::isExecutable(const QString& mimeType)
 }
 
 UrlLoader::UrlLoader(KonqMainWindow *mainWindow, KonqView *view, const QUrl &url, const QString &mimeType, const KonqOpenURLRequest &req, bool trustedSource, bool dontEmbed):
-    QObject(mainWindow), m_mainWindow(mainWindow), m_url(url), m_mimeType(mimeType), m_request(req), m_view(view), m_trustedSource(trustedSource), m_dontEmbed(dontEmbed)
+    QObject(mainWindow), m_mainWindow(mainWindow), m_url(url), m_mimeType(mimeType), m_request(req), m_view(view), m_trustedSource(trustedSource), m_dontEmbed(dontEmbed),
+    m_jobErrorCode(0), m_protocolAllowsReading(KProtocolManager::supportsReading(m_url))
 {
     m_dontPassToWebEnginePart = m_request.args.metaData().contains("DontSendToDefaultHTMLPart");
 }
@@ -105,11 +106,7 @@ void UrlLoader::start()
         }
     }
 
-    if (isMimeTypeKnown(m_mimeType)) {
-        decideAction();
-    } else {
-        m_isAsync = true;
-    }
+    m_isAsync = m_protocolAllowsReading && !isMimeTypeKnown(m_mimeType);
 }
 
 bool UrlLoader::isViewLocked() const
@@ -128,7 +125,13 @@ void UrlLoader::decideAction()
             m_ready = true;
             return;
         default:
-            if (isViewLocked() || shouldEmbedThis()) {
+            if (m_mimeType.isEmpty() && !m_protocolAllowsReading) {
+                //If the protocol doesn't allow reading and we don't have a mimetype associated with it,
+                //use the Open action, as we most likely won't be able to find out the mimetype. This is
+                //what happens, for example, for mailto URLs
+                m_action = OpenUrlAction::Open;
+                return;
+            } else if (isViewLocked() || shouldEmbedThis()) {
                 bool success = decideEmbedOrSave();
                 if (success) {
                     return;
@@ -152,10 +155,12 @@ void UrlLoader::abort()
 
 void UrlLoader::goOn()
 {
-    if (m_ready) {
-        performAction();
+    if (m_isAsync) {
+        launchMimeTypeFinderJob();
     } else {
-        launchOpenUrlJob(true);
+        decideAction();
+        m_ready = true;
+        performAction();
     }
 }
 
@@ -165,22 +170,22 @@ bool UrlLoader::decideEmbedOrSave()
 
     //Use WebEnginePart for konq: URLs even if it's not the default html engine
     if (KonqUrl::hasKonqScheme(m_url)) {
-        m_service = KService::serviceByDesktopName(webEngineName);
+        m_part = findPartById(webEngineName);
     } else {
         //Check whether the view can display the mimetype, but only if the URL hasn't been explicitly
         //typed by the user: in this case, use the preferred service. This is needed to avoid the situation
         //where m_view is a Kate part, the user enters the URL of a web page and the page is opened within
         //the Kate part because it can handle html files.
         if (m_view && m_request.typedUrl.isEmpty() && m_view->supportsMimeType(m_mimeType)) {
-            m_service = m_view->service();
+            m_part = m_view->service();
         } else {
             if (!m_request.serviceName.isEmpty()) {
                 // If the service name has been set by the "--part" command line argument
                 // (detected in handleCommandLine() in konqmain.cpp), then use it as is.
-                m_service = KService::serviceByStorageId(m_request.serviceName);
+                m_part = findPartById(m_request.serviceName);
             } else {
                 // Otherwise, use the preferred service for the MIME type.
-                m_service = KMimeTypeTrader::self()->preferredService(m_mimeType, QStringLiteral("KParts/ReadOnlyPart"));
+                m_part = preferredPart(m_mimeType);
             }
         }
     }
@@ -191,21 +196,26 @@ bool UrlLoader::decideEmbedOrSave()
      * call again this. To avoid this, if the preferred service is webenginepart and m_dontPassToWebEnginePart
      * is true, use the second preferred service (if any); otherwise return false. This will offer the user
      * the option to open or save, instead.
+     *
+     * This can also happen if the URL was opened from a link with the "download" attribute or with a
+     * "CONTENT-DISPOSITION: attachment" header. In these cases, WebEnginePart will always refuse to open
+     * the URL and will ask Konqueror to download it. However, if the preferred part for the URL mimetype is
+     * WebEnginePart itself, this would lead to an endless loop. This check avoids it
      */
-    if (m_dontPassToWebEnginePart && m_service->desktopEntryName() == webEngineName) {
-        KService::List services = KMimeTypeTrader::self()->query(m_mimeType, QStringLiteral("KParts/ReadOnlyPart"));
-        auto findService = [webEngineName](KService::Ptr s){return s->desktopEntryName() != webEngineName;};
-        QList<KService::Ptr>::const_iterator serviceToUse = std::find_if(services.constBegin(), services.constEnd(), findService);
-        if (serviceToUse != services.constEnd()) {
-            m_service = *serviceToUse;
+    if (m_dontPassToWebEnginePart && m_part.pluginId() == webEngineName) {
+        QVector<KPluginMetaData> parts = KParts::PartLoader::partsForMimeType(m_mimeType);
+        auto findPart = [&webEngineName](const KPluginMetaData &md){return md.pluginId() != webEngineName;};
+        QVector<KPluginMetaData>::const_iterator partToUse = std::find_if(parts.constBegin(), parts.constEnd(), findPart);
+        if (partToUse != parts.constEnd()) {
+            m_part = *partToUse;
         } else {
-            m_service = nullptr;
+            m_part = KPluginMetaData();
         }
     }
 
     //If we can't find a service, return false, so that the caller can use decideOpenOrSave to allow the
     //user the possibility of opening the file, since embedding wasn't possibile
-    if (!m_service || !m_service->isValid()) {
+    if (!m_part.isValid()) {
         return false;
     }
 
@@ -219,10 +229,10 @@ bool UrlLoader::decideEmbedOrSave()
     }
 
     if (m_action == OpenUrlAction::Embed) {
-        m_request.serviceName = m_service->desktopEntryName();
+        m_request.serviceName = m_part.pluginId();
     }
 
-    m_ready = m_service || m_action != OpenUrlAction::Embed;
+    m_ready = m_part.isValid() || m_action != OpenUrlAction::Embed;
     return true;
 }
 
@@ -242,7 +252,7 @@ void UrlLoader::decideOpenOrSave()
     m_action = answerWithService.first;
     m_service = answerWithService.second;
     if (m_action == OpenUrlAction::Open && !m_service) {
-        m_service = KApplicationTrader::preferredService(m_mimeType);
+        m_service= KApplicationTrader::preferredService(m_mimeType);
     }
 }
 
@@ -250,7 +260,7 @@ UrlLoader::OpenUrlAction UrlLoader::decideExecute() const {
     if (!m_url.isLocalFile() || !KRun::isExecutable(m_mimeType)) {
         return OpenUrlAction::UnknwonAction;
     }
-    bool canDisplay = !KMimeTypeTrader::self()->query(m_mimeType).isEmpty();
+    bool canDisplay = !KParts::PartLoader::partsForMimeType(m_mimeType).isEmpty();
 
     KMessageBox::ButtonCode code;
     KGuiItem executeGuiItem(i18nc("Execute an executable file", "Execute it"));
@@ -318,29 +328,25 @@ bool UrlLoader::serviceIsKonqueror(KService::Ptr service)
     return service && (service->desktopEntryName() == QLatin1String("konqueror") || service->exec().trimmed().startsWith(QLatin1String("kfmclient")));
 }
 
-void UrlLoader::launchOpenUrlJob(bool pauseOnMimeTypeDetermined)
+void UrlLoader::launchMimeTypeFinderJob()
 {
-    QString mimeType = isMimeTypeKnown(m_mimeType) ? m_mimeType : QString();
-    m_openUrlJob = new KIO::OpenUrlJob(m_url, mimeType, this);
-    m_openUrlJob->setEnableExternalBrowser(false);
-    m_openUrlJob->setRunExecutables(true);
-    m_openUrlJob->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_mainWindow));
-    m_openUrlJob->setSuggestedFileName(m_request.suggestedFileName);
-    m_openUrlJob->setDeleteTemporaryFile(m_request.tempFile);
-    if (pauseOnMimeTypeDetermined) {
-        connect(m_openUrlJob, &KIO::OpenUrlJob::mimeTypeFound, this, &UrlLoader::mimetypeDeterminedByJob);
-    }
-    connect(m_openUrlJob, &KJob::finished, this, &UrlLoader::jobFinished);
-    m_openUrlJob->start();
+    m_mimeTypeFinderJob = new KIO::MimeTypeFinderJob(m_url, this);
+    m_mimeTypeFinderJob->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_mainWindow));
+    m_mimeTypeFinderJob->setSuggestedFileName(m_request.suggestedFileName);
+    connect(m_mimeTypeFinderJob, &KIO::MimeTypeFinderJob::result, this, [this](KJob*){mimetypeDeterminedByJob();});
+    m_mimeTypeFinderJob->start();
 }
 
-void UrlLoader::mimetypeDeterminedByJob(const QString &mimeType)
+void UrlLoader::mimetypeDeterminedByJob()
 {
-    m_mimeType=mimeType;
-    m_openUrlJob->suspend();
-    decideAction();
-    if (m_action != OpenUrlAction::Execute) {
-        m_openUrlJob->kill();
+    if (!m_mimeTypeFinderJob->error()) {
+        m_mimeType=m_mimeTypeFinderJob->mimeType();
+        decideAction();
+    } else {
+        m_jobErrorCode = m_mimeTypeFinderJob->error();
+        m_url = KParts::BrowserRun::makeErrorUrl(m_jobErrorCode, m_mimeTypeFinderJob->errorString(), m_url);
+        m_mimeType = QStringLiteral("text/html");
+        m_action = OpenUrlAction::Embed;
     }
     performAction();
 }
@@ -412,8 +418,13 @@ void UrlLoader::detectSettingsForLocalFiles()
             }
         }
     } else {
-        QMimeDatabase db;
-        m_mimeType = db.mimeTypeForFile(m_url.path()).name();
+        if (QFile::exists(m_url.path())) {
+            QMimeDatabase db;
+            m_mimeType = db.mimeTypeForFile(m_url.path()).name();
+        } else {
+            //Treat the nonexisting file as a directory
+            m_mimeType = QStringLiteral("inode/directory");
+        }
     }
 }
 
@@ -425,6 +436,11 @@ bool UrlLoader::shouldEmbedThis() const
 
 void UrlLoader::embed()
 {
+    if (m_jobErrorCode) {
+        m_url = KParts::BrowserRun::makeErrorUrl(m_jobErrorCode, m_url.scheme(), m_url);
+        m_mimeType = QStringLiteral("text/html");
+        m_part = findPartById(QStringLiteral("webenginepart"));
+    }
     bool embedded = m_mainWindow->openView(m_mimeType, m_url, m_view, m_request);
     if (embedded) {
         done();
@@ -470,11 +486,20 @@ void UrlLoader::open()
     if (m_service && serviceIsKonqueror(m_service) && m_mainWindow->refuseExecutingKonqueror(m_mimeType)) {
         return;
     }
-    KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(m_service);
-    job->setUrls({m_url});
-    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_mainWindow));
-    if (m_request.tempFile) {
-        job->setRunFlags(KIO::ApplicationLauncherJob::DeleteTemporaryFiles);
+
+    KJob *job = nullptr;
+    if (m_service) {
+        KIO::ApplicationLauncherJob *j = new KIO::ApplicationLauncherJob(m_service);
+        j->setUrls({m_url});
+        j->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_mainWindow));
+        if (m_request.tempFile) {
+            j->setRunFlags(KIO::ApplicationLauncherJob::DeleteTemporaryFiles);
+        }
+        job = j;
+    } else {
+        KIO::OpenUrlJob *j = new KIO::OpenUrlJob(m_url);
+        j->setDeleteTemporaryFile(m_request.tempFile);
+        job = j;
     }
     connect(job, &KJob::finished, this, [this, job](){done(job);});
     job->start();
@@ -482,15 +507,14 @@ void UrlLoader::open()
 
 void UrlLoader::execute()
 {
-    //Since only local files can be executed, m_openUrlJob should always be nullptr. However, keep this check, just in case
-    if (!m_openUrlJob) {
-        launchOpenUrlJob(false);
-        connect(m_openUrlJob, &KJob::finished, this, [this](){done(m_openUrlJob);});
-    } else {
-        disconnect(m_openUrlJob, &KJob::finished, this, nullptr); //Otherwise, jobFinished will be called twice
-        connect(m_openUrlJob, &KJob::finished, this, [this](){done(m_openUrlJob);});
-        m_openUrlJob->resume();
-    }
+    m_openUrlJob = new KIO::OpenUrlJob(m_url, m_mimeType, this);
+    m_openUrlJob->setEnableExternalBrowser(false);
+    m_openUrlJob->setRunExecutables(true);
+    m_openUrlJob->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_mainWindow));
+    m_openUrlJob->setSuggestedFileName(m_request.suggestedFileName);
+    m_openUrlJob->setDeleteTemporaryFile(m_request.tempFile);
+    connect(m_openUrlJob, &KJob::finished, this, [this]{done(m_openUrlJob);});
+    m_openUrlJob->start();
 }
 
 //Copied from KParts::BrowserRun::isTextExecutable
@@ -526,8 +550,9 @@ QString UrlLoader::partForLocalFile(const QString& path)
 {
     QMimeDatabase db;
     QString mimetype = db.mimeTypeForFile(path).name();
-    KService::Ptr service = KMimeTypeTrader::self()->preferredService(mimetype, QStringLiteral("KParts/ReadOnlyPart"));
-    return service ? service->name() : QString();
+
+    KPluginMetaData plugin = preferredPart(mimetype);
+    return plugin.pluginId();
 }
 
 UrlLoader::ViewToUse UrlLoader::viewToUse() const
@@ -546,7 +571,7 @@ UrlLoader::ViewToUse UrlLoader::viewToUse() const
 
 void UrlLoader::jobFinished(KJob* job)
 {
-    m_jobHadError = job->error();
+    m_jobErrorCode = job->error();
 }
 
 QDebug operator<<(QDebug dbg, UrlLoader::OpenUrlAction action)
