@@ -1,22 +1,10 @@
 /*
- * This file is part of the KDE project.
- *
- * Copyright (C) 2018 Stefano Crocco <stefano.crocco@alice.it>
- *
- * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation; either version 2.1 of the License, or (at your
- * option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
- * details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- *
- */
+    This file is part of the KDE project.
+
+    SPDX-FileCopyrightText: 2018 Stefano Crocco <stefano.crocco@alice.it>
+
+    SPDX-License-Identifier: LGPL-2.1-or-later
+*/
 
 #ifndef WEBENGINEPARTKIOHANDLER_H
 #define WEBENGINEPARTKIOHANDLER_H
@@ -258,6 +246,16 @@ private:
      * stores it in #m_currentRequest and starts a `KIO::storedGet` for its URL
      */
     void processNextRequest();
+
+    /**
+     * @brief Creates a reply from the error string of the given job
+     *
+     * The reply is stored in m_data
+     *
+     * This function does nothing if the job didn't report an error or if the error string is empty
+     * @param job the job
+     */
+    void createDataFromErrorString(KIO::StoredTransferJob *job);
     
     using RequestJobPointer = QPointer<QWebEngineUrlRequestJob>;
     
